@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: etpan-cfg-storage.c,v 1.13 2004/11/12 13:57:02 hoa Exp $
+ * $Id: etpan-cfg-storage.c,v 1.14 2005/01/30 17:26:59 hoa Exp $
  */
 
 #include "etpan-cfg-storage.h"
@@ -920,11 +920,15 @@ int etpan_storage_config_read(char * filename,
     r = storage_read(entry, &storage);
     if (r != NO_ERROR) {
       char * name;
+#if 0
       char * id;
+#endif
 
+#if 0
       id = etpan_db_entry_get_value(entry, "id");
+#endif
       name = etpan_db_entry_get_value(entry, "name");
-      printf("storage error - %s - %s\n", id, name);
+      printf("storage error - %s\n", name);
       continue;
     }
     
@@ -1023,7 +1027,6 @@ static void write_imap_storage_conf(FILE * f, unsigned int id,
     return;
   
   imap_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = imap\n");
   
@@ -1052,7 +1055,6 @@ static void write_pop3_storage_conf(FILE * f, unsigned int id,
     return;
   
   pop3_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = pop3\n");
   
@@ -1081,7 +1083,6 @@ static void write_nntp_storage_conf(FILE * f, unsigned int id,
     return;
   
   nntp_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = nntp\n");
   
@@ -1109,7 +1110,6 @@ static void write_mbox_storage_conf(FILE * f, unsigned int id,
     return;
   
   mbox_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = mbox\n");
   fprintf(f, "location = %s\n", mbox_data->mbox_pathname);
@@ -1128,7 +1128,6 @@ static void write_mh_storage_conf(FILE * f, unsigned int id,
     return;
   
   mh_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = mh\n");
   fprintf(f, "location = %s\n", mh_data->mh_pathname);
@@ -1147,7 +1146,6 @@ static void write_maildir_storage_conf(FILE * f, unsigned int id,
     return;
   
   maildir_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = maildir\n");
   fprintf(f, "location = %s\n", maildir_data->md_pathname);
@@ -1167,7 +1165,6 @@ static void write_db_storage_conf(FILE * f, unsigned int id,
     return;
   
   db_data = storage->sto_data;
-  fprintf(f, "id = %i\n", id);
   fprintf(f, "name = %s\n", storage->sto_id);
   fprintf(f, "type = db\n");
   fprintf(f, "location = %s\n", db_data->db_pathname);
