@@ -102,6 +102,8 @@ static void free_array(carray * list)
     struct path_info * info;
     
     info = carray_get(list, i);
+    free(info->vpath);
+    free(info->mb);
     free(info);
   }
   carray_free(list);
@@ -754,6 +756,14 @@ struct etpan_discovery_info * etpan_discovery_info_new(int type,
 
 void etpan_discovery_info_free(struct etpan_discovery_info * info)
 {
+  if (info->path_list != NULL) {
+    free_array(info->path_list->mb_list);
+    free(info->path_list);
+  }
+  if (info->updated_path_list != NULL) {
+    free_array(info->updated_path_list->mb_list);
+    free(info->updated_path_list);
+  }
   pthread_mutex_destroy(&info->lock);
   free(info->vpath);
   free(info->start_path);
