@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: etpan-folder-list-app.c,v 1.30 2004/12/16 22:02:20 hoa Exp $
+ * $Id: etpan-folder-list-app.c,v 1.32 2005/02/06 23:48:56 hoa Exp $
  */
 
 #include "etpan-folder-list-app.h"
@@ -56,6 +56,8 @@
 #include "etpan-help-viewer.h"
 #include "etpan-search.h"
 #include "etpan-folder-discover.h"
+#include "etpan-3paned.h"
+#include "etpan-msg-viewer-app.h"
 
 static void idle(struct etpan_subapp * app);
 static void set_fd(struct etpan_subapp * app, fd_set * fds, int * maxfd);
@@ -69,12 +71,14 @@ static void enter(struct etpan_subapp * app, struct etpan_subapp * old_app);
 static void leave(struct etpan_subapp * app, struct etpan_subapp * new_app);
 static int display_init(struct etpan_subapp * app);
 static int get_idle_delay(struct etpan_subapp * app);
+static int get_idle_udelay(struct etpan_subapp * app, struct timeval * delay);
 
 static struct etpan_subapp_driver etpan_folder_list_app_driver = {
   .name = "folder-list",
   .always_handle_key = 0,
   .always_on_top = 0,
   .get_idle_delay = get_idle_delay,
+  .get_idle_udelay = get_idle_udelay,
   .idle = idle,
   .set_fd = set_fd,
   .handle_fd = handle_fd,
@@ -90,12 +94,17 @@ static struct etpan_subapp_driver etpan_folder_list_app_driver = {
   .display_done = NULL,
 };
 
+#define NEW_SELECTION_DELAY (3 * 100000) /* 0.3 s */
+
 struct app_state {
   struct etpan_folder_common_app_state common_state;
   
   /* poll delay */
   time_t last_poll_date;
   int disable_poll;
+  
+  int new_selection;
+  struct timeval last_sel_date;
 };
 
 static int poll_folders(struct etpan_subapp * app);
@@ -180,6 +189,65 @@ static int get_idle_delay(struct etpan_subapp * app)
   return diff;
 }
 
+static int get_idle_udelay(struct etpan_subapp * app, struct timeval * delay)
+{
+  struct app_state * state;
+  struct timeval now;
+  unsigned long d_sec;
+  long d_usec;
+  
+  if (!app->enabled)
+    return -1;
+  
+  state = app->data;
+  
+  if (!state->new_selection)
+    return -1;
+  
+  if (!etpan_3paned(app->app))
+    return -1;
+  
+  gettimeofday(&now, NULL);
+  if (now.tv_sec <= state->last_sel_date.tv_sec)
+    d_sec = (state->last_sel_date.tv_sec - now.tv_sec);
+  else
+    d_sec = 0;
+  d_usec = (state->last_sel_date.tv_usec + NEW_SELECTION_DELAY - now.tv_usec);
+#if 0
+  ETPAN_APP_DEBUG((app->app, "%i %i", d_sec, d_usec));
+#endif
+  if (d_usec > 1000000) {
+    d_sec ++;
+    d_usec -= 1000000;
+  }
+  if (d_usec < 0) {
+    if (d_sec == 0) {
+      d_usec = 0;
+    }
+    else {
+      d_sec --;
+      d_usec += 1000000;
+    }
+  }
+  delay->tv_sec = d_sec;
+  delay->tv_usec = d_usec;
+#if 0
+  ETPAN_APP_DEBUG((app->app, "%i %i", now.tv_sec, now.tv_usec));
+  ETPAN_APP_DEBUG((app->app, "%i %i",
+                      state->last_sel_date.tv_sec,
+                      state->last_sel_date.tv_usec));
+  ETPAN_APP_DEBUG((app->app, "%i %i", d_sec, d_usec));
+#endif
+#if 0
+  ETPAN_APP_DEBUG((app->app, "%i %i", d_sec, d_usec));
+#endif
+  
+  return 0;
+}
+
+static int open_folder(struct etpan_subapp * app,
+    struct mailfolder * folder, int do_switch);
+
 static void idle(struct etpan_subapp * app)
 {
   struct app_state * state;
@@ -190,7 +258,7 @@ static void idle(struct etpan_subapp * app)
     return;
   
   state = app->data;
-
+  
   poll_delay = app->app->config.global_config->poll_delay;
   
   if (poll_delay != 0) {
@@ -205,6 +273,55 @@ static void idle(struct etpan_subapp * app)
       discover_folders(app);
     }
   }
+  
+  if (etpan_3paned(app->app)) {
+    if (state->new_selection) {
+      struct timeval tv_now;
+      unsigned long d_sec;
+      long d_usec;
+    
+      gettimeofday(&tv_now, NULL);
+      if (tv_now.tv_sec <= state->last_sel_date.tv_sec) {
+        d_sec = (state->last_sel_date.tv_sec - tv_now.tv_sec);
+        d_usec = (state->last_sel_date.tv_usec + NEW_SELECTION_DELAY -
+            tv_now.tv_usec);
+        if (d_usec > 1000000) {
+          d_sec ++;
+          d_usec -= 1000000;
+        }
+        if (d_usec < 0) {
+          if (d_sec == 0) {
+            d_usec = 0;
+          }
+          else {
+            d_sec --;
+            d_usec += 1000000;
+          }
+        }
+      }
+      else {
+        d_sec = 0;
+        d_usec = 0;
+      }
+      
+      if ((d_sec == 0) && (d_usec <= 0)) {
+        struct mailfolder * folder;
+      
+        folder = etpan_folder_common_get_selected_folder(&state->common_state);
+        if (folder != NULL) {
+          if (folder->fld_storage != NULL) {
+            int do_poll;
+            
+            do_poll = etpan_vfolder_do_poll(app->app->config.vfolder_config,
+                folder);
+            if (do_poll)
+              open_folder(app, folder, 0);
+          }
+        }
+        state->new_selection = 0;
+      }
+    }
+  }
 }
 
 static void set_fd(struct etpan_subapp * app, fd_set * fds, int * maxfd)
@@ -270,8 +387,8 @@ static int match_folder(struct etpan_subapp * msglist_app, void * data)
   return (etpan_msg_list_app_get_folder(msglist_app) == data);
 }
 
-static int open_folder(struct etpan_subapp * app,
-    struct mailfolder * folder)
+static int ui_basic_open_folder(struct etpan_subapp * app,
+    struct mailfolder * folder, int do_switch)
 {
   struct app_state * state;
   struct etpan_subapp * msglist_app;
@@ -302,12 +419,59 @@ static int open_folder(struct etpan_subapp * app,
   
   /* set folder and run */
   etpan_msg_list_app_set_folder(msglist_app, folder);
-  etpan_app_switch_subapp(msglist_app, 0);
-  ETPAN_APP_DEBUG((app->app, "switch to another folder"));
+  if (do_switch) {
+    etpan_app_switch_subapp(msglist_app, 0);
+    ETPAN_APP_DEBUG((app->app, "switch to another folder"));
+  }
   
   return NO_ERROR;
 }
 
+static int ui_3paned_open_folder(struct etpan_subapp * app,
+    struct mailfolder * folder, int do_switch)
+{
+  struct app_state * state;
+  struct etpan_subapp * msglist_app;
+  struct mailfolder * old_folder;
+  
+  state = app->data;
+  
+  update_folder_info(app, folder);
+  
+  msglist_app = etpan_3paned_get_msglist(app->app);
+  
+  old_folder = etpan_msg_list_app_get_folder(msglist_app);
+  
+  /* set folder and run */
+  
+  if (old_folder != folder) {
+    struct etpan_subapp * msgviewer_app;
+    
+    etpan_msg_list_app_set_folder(msglist_app, folder);
+    ETPAN_APP_DEBUG((app->app, "switch to another folder"));
+    
+    msgviewer_app = etpan_3paned_get_msgview(app->app);
+    etpan_msg_viewer_app_flush(msgviewer_app);
+  }
+  
+  if (do_switch) {
+    etpan_app_switch_subapp(msglist_app, 0);
+  }
+  
+  return NO_ERROR;
+}
+
+static int open_folder(struct etpan_subapp * app,
+    struct mailfolder * folder, int do_switch)
+{
+  if (etpan_3paned(app->app)) {
+    return ui_3paned_open_folder(app, folder, do_switch);
+  }
+  else {
+    return ui_basic_open_folder(app, folder, do_switch);
+  }
+}
+
 static int catchup_messages(struct etpan_subapp * app);
 
 static int clear_stat_folder(struct etpan_subapp * app);
@@ -324,9 +488,13 @@ static void handle_key(struct etpan_subapp * app, int key)
 {
   struct app_state * state;
   struct mailfolder * folder;
+  struct mailfolder * last_selection;
   
   state = app->data;
-
+  
+  last_selection =
+    etpan_folder_common_get_selected_folder(&state->common_state);
+  
   etpan_folder_common_handle_key(app, &state->common_state, key);
 
   switch (key) {
@@ -357,10 +525,19 @@ static void handle_key(struct etpan_subapp * app, int key)
   case 'x':
     clear_stat_folder(app);
     break;
-    
+
   case KEY_CTRL('E'):
     edit_folder_conf(app);
     break;
+    
+  case '\t':
+    if (etpan_3paned(app->app)) {
+      struct etpan_subapp * subapp;
+      
+      subapp = etpan_3paned_get_msglist(app->app);
+      etpan_app_switch_subapp(subapp, 0);
+    }
+    break;
   }
   
   if (!state->disable_poll) {
@@ -370,7 +547,7 @@ static void handle_key(struct etpan_subapp * app, int key)
       folder = etpan_folder_common_get_selected_folder(&state->common_state);
       if (folder != NULL) {
         if (folder->fld_storage != NULL)
-          open_folder(app, folder);
+          open_folder(app, folder, 1);
       }
       break;
       
@@ -387,6 +564,12 @@ static void handle_key(struct etpan_subapp * app, int key)
       break;
     }
   }
+  
+  folder = etpan_folder_common_get_selected_folder(&state->common_state);
+  if (folder != last_selection) {
+    gettimeofday(&state->last_sel_date, NULL);
+    state->new_selection = 1;
+  }
 }
 
 static void set_title(struct etpan_subapp * app);
@@ -458,6 +641,8 @@ int etpan_folder_list_set_root_folder(struct etpan_subapp * app,
     struct mailfolder * root)
 {
   struct app_state * state;
+  unsigned int i;
+  int r;
   
   state = app->data;
   
@@ -465,8 +650,24 @@ int etpan_folder_list_set_root_folder(struct etpan_subapp * app,
 #if 0
   state->state = STATE_NORMAL;
 #endif
-  return etpan_folder_common_set_root_folder(&state->common_state, root,
+  gettimeofday(&state->last_sel_date, NULL);
+  state->new_selection = 1;
+  
+  r = etpan_folder_common_set_root_folder(&state->common_state, root,
       ETPAN_FOLDER_COMMON_SHOW_STAT);
+
+  if (state->common_state.folder_tab != NULL) {
+    for (i = 0 ; i < carray_count(state->common_state.folder_tab) ; i ++) {
+      struct mailfolder * folder;
+      
+      folder = carray_get(state->common_state.folder_tab, i);
+      
+      if (folder == app->app->config.vfolder_config->default_folder)
+        state->common_state.selected = i;
+    }
+  }
+  
+  return r;
 }
 
 struct etpan_subapp * etpan_folder_list_app_new(struct etpan_app * app)
@@ -950,7 +1151,7 @@ int etpan_folder_list_next_unread_message(struct etpan_subapp * app,
   msglist_app = etpan_app_find_subapp(app->app, "msg-list",
       1, match_folder, folder);
   
-  open_folder(app, folder);
+  open_folder(app, folder, 1);
   
   if (msglist_app != NULL) {
     etpan_msg_list_first_unread(msglist_app, 1);
@@ -982,7 +1183,8 @@ view the content.\n\
                  we already requested the count\n\
 - Ctrl-C      : mark all messages as read in the selected folder\n\
 \n\
-- /           : search messages\n\
+- /           : search folder by name\n\
+- :           : search messages\n\
 \n\
 - [Enter]     : view folder content\n\
 - m           : compose new message\n\
