@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: etpan-msg-list-app.c,v 1.30 2004/12/27 15:27:38 hoa Exp $
+ * $Id: etpan-msg-list-app.c,v 1.32 2005/02/06 23:48:56 hoa Exp $
  */
 
 #include "etpan-msg-list-app.h"
@@ -64,6 +64,7 @@
 #include "etpan-mime-tools.h"
 #include "etpan-help-viewer.h"
 #include "etpan-search.h"
+#include "etpan-3paned.h"
 
 static void idle(struct etpan_subapp * app);
 static void set_fd(struct etpan_subapp * app, fd_set * fds, int * maxfd);
@@ -77,12 +78,14 @@ static void enter(struct etpan_subapp * app, struct etpan_subapp * old_app);
 static void leave(struct etpan_subapp * app, struct etpan_subapp * new_app);
 static int display_init(struct etpan_subapp * app);
 static int get_idle_delay(struct etpan_subapp * app);
+static int get_idle_udelay(struct etpan_subapp * app, struct timeval * delay);
 
 static struct etpan_subapp_driver etpan_msg_list_app_driver = {
   .name = "msg-list",
   .always_handle_key = 0,
   .always_on_top = 0,
   .get_idle_delay = get_idle_delay,
+  .get_idle_udelay = get_idle_udelay,
   .idle = idle,
   .set_fd = set_fd,
   .handle_fd = handle_fd,
@@ -134,8 +137,13 @@ struct app_state {
   char * path_description;
   
   time_t last_poll_date;
+  
+  int new_selection;
+  struct timeval last_sel_date;
 };
 
+#define NEW_SELECTION_DELAY (3 * 100000) /* 0.3 s */
+
 static void set_title(struct etpan_subapp * app);
 
 static int get_msglist(struct etpan_subapp * app);
@@ -169,6 +177,61 @@ static int get_idle_delay(struct etpan_subapp * app)
   return diff;
 }
 
+static int get_idle_udelay(struct etpan_subapp * app, struct timeval * delay)
+{
+  struct app_state * state;
+  struct timeval now;
+  unsigned long d_sec;
+  long d_usec;
+  
+  if (!app->enabled)
+    return -1;
+  
+  state = app->data;
+  
+  if (!state->new_selection)
+    return -1;
+  
+  if (!etpan_3paned(app->app))
+    return -1;
+  
+  gettimeofday(&now, NULL);
+  if (now.tv_sec <= state->last_sel_date.tv_sec)
+    d_sec = (state->last_sel_date.tv_sec - now.tv_sec);
+  else
+    d_sec = 0;
+  d_usec = (state->last_sel_date.tv_usec + NEW_SELECTION_DELAY - now.tv_usec);
+  if (d_usec > 1000000) {
+    d_sec ++;
+    d_usec -= 1000000;
+  }
+  if (d_usec < 0) {
+    if (d_sec == 0) {
+      d_usec = 0;
+    }
+    else {
+      d_sec --;
+      d_usec += 1000000;
+    }
+  }
+  delay->tv_sec = d_sec;
+  delay->tv_usec = d_usec;
+#if 0
+  ETPAN_APP_DEBUG((app->app, "%i %i", now.tv_sec, now.tv_usec));
+  ETPAN_APP_DEBUG((app->app, "%i %i",
+                      state->last_sel_date.tv_sec,
+                      state->last_sel_date.tv_usec));
+  ETPAN_APP_DEBUG((app->app, "%i %i", d_sec, d_usec));
+#endif
+#if 0
+  ETPAN_APP_DEBUG((app->app, "%i %i", d_sec, d_usec));
+#endif
+  
+  return 0;
+}
+
+static int view_message(struct etpan_subapp * app, int do_switch);
+
 static void idle(struct etpan_subapp * app)
 {
   struct app_state * state;
@@ -196,6 +259,43 @@ static void idle(struct etpan_subapp * app)
       get_msglist(app);
     }
   }
+
+  if (etpan_3paned(app->app)) {
+    if (state->new_selection) {
+      struct timeval tv_now;
+      unsigned long d_sec;
+      long d_usec;
+    
+      gettimeofday(&tv_now, NULL);
+      if (tv_now.tv_sec <= state->last_sel_date.tv_sec) {
+        d_sec = (state->last_sel_date.tv_sec - tv_now.tv_sec);
+        d_usec = (state->last_sel_date.tv_usec + NEW_SELECTION_DELAY -
+            tv_now.tv_usec);
+        if (d_usec > 1000000) {
+          d_sec ++;
+          d_usec -= 1000000;
+        }
+        if (d_usec < 0) {
+          if (d_sec == 0) {
+            d_usec = 0;
+          }
+          else {
+            d_sec --;
+            d_usec += 1000000;
+          }
+        }
+      }
+      else {
+        d_sec = 0;
+        d_usec = 0;
+      }
+    
+      if ((d_sec == 0) && (d_usec <= 0)) {
+        view_message(app, 0);
+        state->new_selection = 0;
+      }
+    }
+  }
 }
 
 static void free_msg_list(struct etpan_subapp * subapp);
@@ -227,10 +327,10 @@ static int msgviewer_match_msg(struct etpan_subapp * app, void * data)
 }
 
 
-static int open_viewer(struct etpan_subapp * app,
+static int ui_basic_open_viewer(struct etpan_subapp * app,
     struct mailfolder * folder,
     mailmessage * msg,
-    char * filename, FILE * f, carray * attr)
+    char * filename, FILE * f, carray * attr, int do_switch)
 {
   struct app_state * state;
   struct etpan_subapp * msgviewer_app;
@@ -262,11 +362,51 @@ static int open_viewer(struct etpan_subapp * app,
   if (r != NO_ERROR)
     return r;
   
-  etpan_app_switch_subapp(msgviewer_app, 0);
+  if (do_switch)
+    etpan_app_switch_subapp(msgviewer_app, 0);
   
   return NO_ERROR;
 }
 
+static int ui_3paned_open_viewer(struct etpan_subapp * app,
+    struct mailfolder * folder,
+    mailmessage * msg,
+    char * filename, FILE * f, carray * attr, int do_switch)
+{
+  struct app_state * state;
+  struct etpan_subapp * msgviewer_app;
+  int r;
+  
+  state = app->data;
+  
+  msgviewer_app = etpan_3paned_get_msgview(app->app);
+  
+  r = etpan_msg_viewer_app_set_msg(msgviewer_app, folder, msg,
+    filename, f, attr);
+  if (r != NO_ERROR)
+    return r;
+  
+  if (do_switch)
+    etpan_app_switch_subapp(msgviewer_app, 0);
+  
+  return NO_ERROR;
+}
+
+static int open_viewer(struct etpan_subapp * app,
+    struct mailfolder * folder,
+    mailmessage * msg,
+    char * filename, FILE * f, carray * attr, int do_switch)
+{
+  if (etpan_3paned(app->app)) {
+    return ui_3paned_open_viewer(app, folder,
+        msg, filename, f, attr, do_switch);
+  }
+  else {
+    return ui_basic_open_viewer(app, folder,
+        msg, filename, f, attr, do_switch);
+  }
+}
+
 
 static int mimelist_match_msg(struct etpan_subapp * mimelist_app, void * data)
 {
@@ -419,11 +559,21 @@ static void thread_get_msg_list_callback(struct etpan_subapp * app,
           state->selected);
     }
   }
+  
+  if (etpan_3paned(app->app)) {
+    gettimeofday(&state->last_sel_date, NULL);
+    state->new_selection = 1;
+  }
 }
 
+struct etpan_render_info {
+  int do_switch;
+};
+
 static void thread_render_handle_cancel(struct etpan_subapp * app,
     struct etpan_thread_op * op, int app_op_type, void * data)
 {
+  free(data);
   etpan_queue_unref_msg_mime(app, op->data.mailaccess.msg);
 }
 
@@ -435,7 +585,13 @@ static void thread_render_callback(struct etpan_subapp * app,
 {
   struct etpan_message_render_result * result;
   int r;
-
+  struct etpan_render_info * render_info_data;
+  int do_switch;
+  
+  render_info_data = data;
+  do_switch = render_info_data->do_switch;
+  free(data);
+  
   if (op->err != NO_ERROR) {
     mailmessage * msg;
     
@@ -459,7 +615,8 @@ static void thread_render_callback(struct etpan_subapp * app,
   /* give the message to the viewer */
   
   r = open_viewer(app, op->data.mailaccess.folder,
-      op->data.mailaccess.msg, result->filename, result->f, result->attr);
+      op->data.mailaccess.msg, result->filename, result->f, result->attr,
+      do_switch);
   
   if (r != NO_ERROR) {
     fclose(result->f);
@@ -540,8 +697,6 @@ static int search_messages(struct etpan_subapp * app);
 
 static int quick_search(struct etpan_subapp * app);
 
-static int view_message(struct etpan_subapp * app);
-
 static int view_mime(struct etpan_subapp * app);
 
 static int reply_message(struct etpan_subapp * app, int reply_type);
@@ -590,9 +745,12 @@ static int show_help(struct etpan_subapp * app);
 static void handle_key(struct etpan_subapp * app, int key)
 {
   struct app_state * state;
+  unsigned int last_selection;
   
   state = app->data;
 
+  last_selection = state->selected;
+  
   switch (key) {
   case '0':
   case '1':
@@ -691,13 +849,30 @@ static void handle_key(struct etpan_subapp * app, int key)
     break;
     
   case 'q':
+    if (etpan_3paned(app->app)) {
+      etpan_app_quit_subapp(etpan_3paned_get_folderlist(app->app));
+    }
+    else {
+      etpan_app_quit_subapp(app);
+    }
+    break;
+    
   case KEY_LEFT:
-    etpan_app_quit_subapp(app);
+    if (etpan_3paned(app->app)) {
+      struct etpan_subapp * folderlist_app;
+      
+      folderlist_app = etpan_3paned_get_folderlist(app->app);
+      
+      etpan_app_switch_subapp(folderlist_app, 0);
+    }
+    else {
+      etpan_app_quit_subapp(app);
+    }
     break;
     
   case '\n':
   case KEY_RIGHT:
-    view_message(app);
+    view_message(app, 1);
     break;
 
   case 'v':
@@ -785,10 +960,12 @@ static void handle_key(struct etpan_subapp * app, int key)
   case KEY_CTRL('A'):
     select_all(app, 1);
     break;
-    
+
+#if 0    
   case KEY_CTRL('I'):
     select_all(app, 0);
     break;
+#endif
     
   case KEY_CTRL('U'):
     select_all(app, -1);
@@ -833,6 +1010,27 @@ static void handle_key(struct etpan_subapp * app, int key)
   case '?':
     show_help(app);
     break;
+
+  case '\t':
+    if (etpan_3paned(app->app)) {
+      struct etpan_subapp * subapp;
+      
+      subapp = etpan_3paned_get_msgview(app->app);
+      etpan_app_switch_subapp(subapp, 0);
+    }
+    break;
+  }
+  
+  if (etpan_3paned(app->app)) {
+    struct etpan_subapp * msgviewer_app;
+    
+    if (last_selection != state->selected) {
+      gettimeofday(&state->last_sel_date, NULL);
+      state->new_selection = 1;
+      
+      msgviewer_app = etpan_3paned_get_msgview(app->app);
+      etpan_msg_viewer_app_flush(msgviewer_app);
+    }
   }
 }
 
@@ -2163,7 +2361,7 @@ mailmessage * etpan_msg_list_app_get_selected_msg(struct etpan_subapp * app)
   return tree->node_msg;
 }
 
-static int view_message(struct etpan_subapp * app)
+static int view_message(struct etpan_subapp * app, int do_switch)
 {
   struct app_state * state;
   mailmessage * msg;
@@ -2188,23 +2386,34 @@ static int view_message(struct etpan_subapp * app)
     etpan_app_switch_subapp(msgviewer_app, 0);
   }
   else {
+    struct etpan_render_info * render_info_data;
+    
     if (msg->msg_uid != NULL)
       ETPAN_APP_LOG((app->app, "retrieving message %i, %s",
                         msg->msg_index, msg->msg_uid));
     else
       ETPAN_APP_LOG((app->app, "retrieving message %i", msg->msg_index));
 
+    render_info_data = malloc(sizeof(* render_info_data));
+    if (render_info_data == NULL) {
+      ETPAN_APP_LOG((app->app, "view message - not enough memory"));
+      res = ERROR_MEMORY;
+      goto err;
+    }
+    render_info_data->do_switch = do_switch;
+    
     r = etpan_queue_ref_msg_mime(app, msg);
     if (r != NO_ERROR) {
       ETPAN_APP_LOG((app->app, "view message - not enough memory"));
       res = r;
       goto err;
     }
+    
     r = etpan_subapp_thread_msg_op_add(app,
         THREAD_ID_MSGLIST_RENDER_MSG, ETPAN_THREAD_MESSAGE_RENDER_MIME,
         msg, NULL,
         NULL,
-        thread_render_callback, NULL, thread_render_handle_cancel);
+        thread_render_callback, render_info_data, thread_render_handle_cancel);
     if (r != NO_ERROR) {
       etpan_queue_unref_msg_mime(app, msg);
       ETPAN_APP_LOG((app->app, "view message - not enough memory"));
@@ -2862,7 +3071,7 @@ int etpan_msg_list_app_open_next(struct etpan_subapp * app,
   r = next_unread(app, forward, 1, 1);
   if (r == NO_ERROR) {
     etpan_msg_list_update_view(app);
-    r = view_message(app);
+    r = view_message(app, 1);
     
     return r;
   }
