@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: etpan-msg-new.c,v 1.20 2004/12/16 15:54:15 hoa Exp $
+ * $Id: etpan-msg-new.c,v 1.21 2005/01/29 15:46:53 hoa Exp $
  */
 
 #include "etpan-msg-new.h"
@@ -788,6 +788,7 @@ int etpan_message_send(struct etpan_subapp * app,
   int send_type;
   int r;
   struct mailfolder * sent_folder;
+  struct etpan_message_append_arg * append_arg;
   
   send_type = ETPAN_SEND_FLAG_NONE;
   if (fields != NULL) {
@@ -805,6 +806,7 @@ int etpan_message_send(struct etpan_subapp * app,
   if (sent_folder != NULL) {
     char * dup_filename;
     struct mailfolder * folder;
+    struct mail_flags * flags;
     
     /* duplicate the file because the thread will remove the file at the end */
     dup_filename = etpan_dup_file(filename);
@@ -815,16 +817,29 @@ int etpan_message_send(struct etpan_subapp * app,
     
     folder = sent_folder;
     
+    append_arg = malloc(sizeof(* append_arg));
+    if (append_arg == NULL) {
+      ETPAN_APP_LOG((app->app, "message postpone - not enough memory"));
+      free(dup_filename);
+      return ERROR_MEMORY;
+    }
+    
+    flags = mail_flags_new_empty();
+    flags->fl_flags = MAIL_FLAG_SEEN;
+    append_arg->filename = dup_filename;
+    append_arg->flags = flags;
+    
     r = etpan_subapp_thread_folder_op_add(app, THREAD_ID_MIMEEDIT_SEND,
         ETPAN_THREAD_FOLDER_APPEND_MESSAGE,
         folder,
         NULL, NULL,
-        dup_filename,
+        append_arg,
         NULL, NULL, NULL);
     if (r != NO_ERROR) {
       ETPAN_APP_LOG((app->app, "message send - not enough memory"));
       unlink(dup_filename);
       free(dup_filename);
+      free(append_arg);
       return ERROR_MEMORY;
     }
   }
@@ -843,16 +858,27 @@ int etpan_message_send(struct etpan_subapp * app,
     
     folder = post_from_folder;
     
+    append_arg = malloc(sizeof(* append_arg));
+    if (append_arg == NULL) {
+      ETPAN_APP_LOG((app->app, "message postpone - not enough memory"));
+      free(dup_filename);
+      return ERROR_MEMORY;
+    }
+    
+    append_arg->filename = dup_filename;
+    append_arg->flags = NULL;
+    
     r = etpan_subapp_thread_folder_op_add(app, THREAD_ID_MIMEEDIT_SEND,
         ETPAN_THREAD_FOLDER_APPEND_MESSAGE,
         folder,
         NULL, NULL,
-        dup_filename,
+        append_arg,
         NULL, NULL, NULL);
     if (r != NO_ERROR) {
       ETPAN_APP_LOG((app->app, "message send - not enough memory"));
       unlink(dup_filename);
       free(dup_filename);
+      free(append_arg);
       return ERROR_MEMORY;
     }
   }
